@@ -8,7 +8,8 @@
 #define PRNTHR 1000
 #define NPTOTOL 3000
 #define NGRID 10
-#define NSIM 128.
+#define PI 3.14159265359
+/* #define NSIM 512. */
 
 typedef struct Zone {
   realT prob; /* Halo probability */
@@ -58,6 +59,7 @@ int main(int argc,char **argv) {
 
   FILE *pos, *inz, *outz, *outztmp, *outlist;
   int i, j, h, h2,h3,c, np, nhaloes, nhunb;
+  int nsim;
   int *hp, niter;
   char *unb;
   realT e1;
@@ -93,17 +95,18 @@ int main(int argc,char **argv) {
 
   halready = 0;
   
-  if (argc != 10) {
+  if (argc != 11) {
     printf("Wrong number of arguments.\n");
     printf("arg1: box size\n");
-    printf("arg2: Omega_matter\n");
-    printf("arg3: scale factor a\n");
-    printf("arg4: position file\n");
-    printf("arg5: velocity file\n");
-    printf("arg6: input zone file\n");
-    printf("arg7: output bound zone file\n");
-    printf("arg8: output text file\n");
-    printf("arg9: unbinding f\n\n");
+    printf("arg2: nsim\n");
+    printf("arg3: Omega_matter\n");
+    printf("arg4: scale factor a\n");
+    printf("arg5: position file\n");
+    printf("arg6: velocity file\n");
+    printf("arg7: input zone file\n");
+    printf("arg8: output bound zone file\n");
+    printf("arg9: output text file\n");
+    printf("arg10: unbinding f\n\n");
     exit(0);
   }
   if (sscanf(argv[1],"%"vozRealSym,&boxsize) != 1) {
@@ -111,38 +114,42 @@ int main(int argc,char **argv) {
     exit(0);
   }
   boxsize2 = boxsize/2.;
-  if (sscanf(argv[2],"%"vozRealSym,&omegam) != 1) {
-    printf("%s is not a valid Omega_matter\n",argv[2]);
+  if (sscanf(argv[2],"%d",&nsim) != 1) {
+    printf("%s is not a valid Nsim\n",argv[2]);
     exit(0);
   }
-  if (sscanf(argv[3],"%"vozRealSym,&a) != 1) {
-    printf("%s is not a valid scale factor\n",argv[3]);
+  if (sscanf(argv[3],"%"vozRealSym,&omegam) != 1) {
+    printf("%s is not a valid Omega_matter\n",argv[3]);
+    exit(0);
+  }
+  if (sscanf(argv[4],"%"vozRealSym,&a) != 1) {
+    printf("%s is not a valid scale factor\n",argv[4]);
     exit(0);
   }  
-  posfile = argv[4];
-  velfile = argv[5];
-  inzfile = argv[6];
-  outzfile = argv[7];
-  outlistfile = argv[8];
-  if (sscanf(argv[9],"%"vozRealSym,&potfact) != 1) {
-    printf("%s is not a valid potfact\n",argv[9]);
+  posfile = argv[5];
+  velfile = argv[6];
+  inzfile = argv[7];
+  outzfile = argv[8];
+  outlistfile = argv[9];
+  if (sscanf(argv[10],"%"vozRealSym,&potfact) != 1) {
+    printf("%s is not a valid potfact\n",argv[10]);
     exit(0);
   }
 
-  cell = boxsize/NSIM;
+  cell = boxsize/nsim;
   hubble = 100. * sqrt(omegam/(a*a*a) + 1. - omegam);
   /* Assumes Omega_matter + Omega_lambda = 1 */
   /* There's no little_h because distance measures are already in Mpc/h */
   printf("a = %g => Multiplying H0 by %e\n",a,hubble/100.);
   /* When A != 1 */
-  potconstreal = omegam * 3.*cell*cell*cell*100.*100./(8.*3.1415927);
+  potconstreal = omegam * 3.*cell*cell*cell*100.*100./(8.*PI);
   /* GM_particle, plus conversion factors to (km/sec)^2 */
 
   printf("cell, potconst = %g,%e\n",cell,potconstreal);
 
   /* Read positions & velocities */
-  np = posread(posfile,&p,cell);
-  if (velread(velfile,&v,1.) != np) {
+  np = posread(posfile,&p,boxsize);
+  if (velread(velfile,&v,1.0) != np) {
     printf("Numbers of particles don't match! Exiting.\n");
     exit(0);
   }
