@@ -1,19 +1,25 @@
 /* jozov.c by Mark Neyrinck
-
-   This program uses the tessellation from the voz* programs
-   to produce the final ZOBOV result
+    This program uses the tessellation from the voz* programs
+    to produce ZOBOV results (possibly, subject to subsequent 
+    post-processing with zobovpostproc.py)
 */
+
+
+/* jovoz.c by Mark Neyrinck */
 #include <stdio.h>
 #include <stdlib.h>
 #include <math.h>
+#include <string.h>
 #include "voz.h"
 
-#define NLINKS 300 /* Number of possible links with the same rho_sl */
+#define NLINKS 30000 /* Number of possible links with the same rho_sl */
 #define FF fflush(stdout)
+#define FNL 256 /* Max length of filenames */
 
 typedef struct Particle {
   realT dens;
   int nadj;
+  int nadj_count;
   int *adj;
 } PARTICLE;
 
@@ -41,53 +47,80 @@ void findrtop(realT *a, int na, int *iord, int nb);
 
 int main(int argc,char **argv) {
 
-  FILE *adj, *vol, *zon, *zon2, *txt;
+  FILE *adj, *vol, *zon, *vod, *txt;
   PARTICLE *p;
   ZONE *z;
   ZONET *zt;
-  char *adjfile, *volfile, *zonfile, *zonfile2, *txtfile;
+  char *prefix, adjfile[FNL], volfile[FNL], zonefile[FNL], voidfile[FNL], txtfile[FNL];
   int i, j,k,l, h, h2,hl,n,np, np2, nzones, nhl, nhlcount, nhl2;
   int *jumper, *jumped, *numinh;
   int *zonenum, *zonelist, *zonelist2;
   int link[NLINKS], link2, nl;
-  realT lowvol, voltol, prob;
+  realT lowvol, voltol, borderdens, invborderdens, prob;
 
   int q,q2;
 
   int za, nin;
   int testpart;
-  char already, interior, *inyet, *inyet2, added, beaten;
+  char already, interior, *inyet, *inyet2, added, beaten, obo;
   int *nm, **m;
 
-  realT maxvol, minvol;
+  realT maxdens, maxdens_border, mindens, mindens_border;
   realT *sorter, e1,maxdenscontrast;
   int *iord;
 
   e1 = exp(1.)-1.;
 
-  if (argc != 7) {
+  if (argc != 5) {
     printf("Wrong number of arguments.\n");
-    printf("arg1: adjacency file\n");
-    printf("arg2: volume file\n");
-    printf("arg3: output file containing particles in each zone\n");
-    printf("arg4: output file containing zones in each void\n");
-    printf("arg5: output text file\n");
-    printf("arg6: Density threshold (0 for no threshold)\n\n");
+    printf("arg1: (ZOBOV) (v)oid or (VOBOZ) (c)luster-finding?\n");
+    printf("arg2: run prefix\n");
+    printf("arg3: Density threshold (0 for no threshold)\n");
+    printf("arg4: Border-particle density (0 for 0.9e30 (voids), 1.1e-30 (clusters))\n\n");
     exit(0);
   }
-  adjfile = argv[1];
-  volfile = argv[2];
-  zonfile = argv[3];
-  zonfile2 = argv[4];
-  txtfile = argv[5];
-  if (sscanf(argv[6],"%"vozRealSym,&voltol) == 0) {
+  if (sscanf(argv[1],"%c",&obo) == 0) {
+    printf("Bad density threshold.\n");
+    exit(0);
+  }  
+  obo = tolower(obo);
+  if (obo == 'c'){
+    printf("Finding clusters.\n");
+  }
+  else if (obo == 'v'){
+    printf("Finding voids.\n");
+  }
+  else{
+    printf("1st arg: please enter v for voids or c for clusters.\n\n");
+    exit(0);
+  }
+  prefix = argv[2];
+
+  sprintf(adjfile,"%s.adj",prefix);
+  sprintf(volfile,"%s.vol",prefix);
+  if (sscanf(argv[3],"%"vozRealSym,&voltol) == 0) {
+    printf("Bad density threshold.\n");
+    exit(0);
+  }  
+  if (sscanf(argv[4],"%"vozRealSym,&borderdens) == 0) {
     printf("Bad density threshold.\n");
     exit(0);
   }
   if (voltol <= 0.) {
     printf("Proceeding without a density threshold.\n");
+    sprintf(zonefile,"%s.zone",prefix);
+    sprintf(voidfile,"%s.void",prefix);
+    sprintf(txtfile,"%s.txt",prefix);
     voltol = 1e30;
+  } else {
+    sprintf(zonefile,"%s.%f.zone",prefix,voltol);
+    sprintf(voidfile,"%s.%f.void",prefix,voltol);
+    sprintf(txtfile,"%s.%f.txt",prefix,voltol);
   }
+  if (borderdens == 0.) {
+    borderdens = 1.1e-30;
+  }
+  invborderdens = 1./borderdens;
 
   adj = fopen(adjfile, "r");
   if (adj == NULL) {
@@ -100,13 +133,16 @@ int main(int argc,char **argv) {
   FF;
 
   p = (PARTICLE *)malloc(np*sizeof(PARTICLE));
+  printf("adj: %d particles\n", np);
+  FF;
   /* Adjacencies*/
   for (i=0;i<np;i++) {
     fread(&p[i].nadj,1,sizeof(int),adj); 
     /* The number of adjacencies per particle */
-    if (p[i].nadj > 0)
+    if (p[i].nadj > 0) {
       p[i].adj = (int *)malloc(p[i].nadj*sizeof(int));
-    p[i].nadj = 0; /* Temporarily, it's an adj counter */
+    }
+    p[i].nadj_count = 0;
   }
   for (i=0;i<np;i++) {
     fread(&nin,1,sizeof(int),adj);
@@ -115,11 +151,15 @@ int main(int argc,char **argv) {
 	fread(&j,1,sizeof(int),adj);
 	if (j < np) {
 	  /* Set both halves of the pair */
-	  p[i].adj[p[i].nadj] = j;
-	  p[j].adj[p[j].nadj] = i;
-	  p[i].nadj++; p[j].nadj++;
+	  if ((p[i].nadj_count < p[i].nadj) && (p[j].nadj_count < p[j].nadj)){
+	    p[i].adj[p[i].nadj_count] = j;
+	    p[j].adj[p[j].nadj_count] = i;
+	    p[i].nadj_count++; p[j].nadj_count++;
+	  } else{
+	    printf("input file possibly weird; extradj for %d or %d\n",i,j);
+	  }
 	} else {
-	  printf("%d: adj = %d\n",i,j);
+	  printf("adj(%d)=%d>np\n",i,j); FF;
 	}
       }
   }
@@ -156,7 +196,12 @@ int main(int argc,char **argv) {
       printf("Whacked-out volume found, of particle %d: %g\n",i,p[i].dens);
       p[i].dens = 1.;
     }
-    p[i].dens = 1./p[i].dens; /* Get density from volume */
+    if (obo == 'v'){
+      p[i].dens = 1./p[i].dens; /* Get density from volume, if we're looking for voids */
+    }
+    if ((obo == 'c') && (p[i].dens < borderdens)) {
+      p[i].dens = 1./p[i].dens; /* high-density border particles ->  low-density border particles*/
+    }
   }
   fclose(vol);
 
@@ -166,11 +211,11 @@ int main(int argc,char **argv) {
 
   /* find jumper */
   for (i = 0; i < np; i++) {
-    minvol = p[i].dens; jumper[i] = -1;
+    mindens = p[i].dens; jumper[i] = -1;
     for (j=0; j< p[i].nadj; j++) {
-      if (p[p[i].adj[j]].dens < minvol) {
+      if (p[p[i].adj[j]].dens < mindens) {
 	jumper[i] = p[i].adj[j];
-	minvol = p[jumper[i]].dens;
+	mindens = p[jumper[i]].dens;
       }
     }
     numinh[i] = 0;
@@ -301,48 +346,40 @@ int main(int argc,char **argv) {
     free(zt[h].adj);
     free(zt[h].slv);
     z[h].np = numinh[z[h].core];
-  }
-  free(zt);
-  free(numinh);
-
-  m = (int **)malloc(nzones*sizeof(int *));
-  /* Not in the zone struct since it'll be freed up (contiguously, we hope)
-     soon */
-  nm = (int *)malloc(nzones*sizeof(int));
-  for (h=0; h<nzones; h++) {
-    m[h] = (int *)malloc(z[h].np*sizeof(int));
-    nm[h] = 0;
     z[h].vol = 0.;
   }
+  free(zt);
   for (i=0; i<np; i++) {
     h = zonenum[jumped[i]];
-    if (i == z[h].core) {
-      m[h][nm[h]] = m[h][0];
-      m[h][0] = i; /* Put the core particle at the top of the list */
+    if ((obo == 'v')&&(p[i].dens > borderdens)) {
+      z[h].vol += 1.0/(realT)p[i].dens;
+    }
+    else if ((obo == 'c')&&(p[i].dens < invborderdens)) {
+      z[h].vol += p[i].dens;
     } else {
-      m[h][nm[h]] = i;
+      /* remove border particles from np's */
+      z[h].np --;
     }
-    z[h].vol += 1.0/(realT)p[i].dens;
-    nm[h] ++;
   }
-  free(nm);
+  free(numinh);
 
-
-  zon = fopen(zonfile,"w");
+  zon = fopen(zonefile,"w");
   if (zon == NULL) {
-    printf("Problem opening zonefile %s.\n\n",zonfile);
+    printf("Problem opening zonefile %s.\n\n",zonefile);
     exit(0);
   }
-  fwrite(&np,1,sizeof(int),zon);
-  fwrite(&nzones,1,sizeof(int),zon);
-  for (h=0; h<nzones; h++) {
-    fwrite(&(z[h].np),1,sizeof(int),zon);
-    fwrite(m[h],z[h].np,sizeof(int),zon);
-    free(m[h]);
+  fprintf(zon,"%d %d\n",np,nzones);
+  for (i=0; i<np; i++) {
+    if (((obo == 'v')&&(p[i].dens > borderdens)) ||
+	((obo == 'c')&&(p[i].dens < invborderdens))) {
+      fprintf(zon, "%d\n",zonenum[jumped[i]]);
+    } else { /* Border particle */
+      fprintf(zon, "-1\n");
+    }
   }
-  free(m);
+  
   close(zon);
-
+  
   inyet = (char *)malloc(nzones*sizeof(char));
   inyet2 = (char *)malloc(nzones*sizeof(char));
   zonelist = (int *)malloc(nzones*sizeof(int));
@@ -356,23 +393,32 @@ int main(int argc,char **argv) {
 
   nhl = 0; 
 
-  maxvol = 0.;
-  minvol = BF;
+  maxdens = -BF;
+  maxdens_border = -BF;
+  mindens = BF;
+  mindens_border = BF;
   maxdenscontrast = 0.;
   for(i=0;i<np; i++){
-    if (p[i].dens > maxvol) maxvol = p[i].dens;
-    if (p[i].dens < minvol) minvol = p[i].dens;
+    if ((p[i].dens > maxdens)&&(p[i].dens < invborderdens)) maxdens = p[i].dens;
+    if (p[i].dens > maxdens_border) maxdens_border = p[i].dens;
+    if ((p[i].dens < mindens)&&(p[i].dens > borderdens)) mindens = p[i].dens;
+    if (p[i].dens < mindens_border) mindens_border = p[i].dens;
   }
-  printf("Densities range from %e to %e.\n",minvol,maxvol);FF;
+  printf("Densities range from %e (%e w/border) to %e (%e w/border).\n",mindens,mindens_border,maxdens,maxdens_border);FF;
 
-  zon2 = fopen(zonfile2,"w");
-  if (zon2 == NULL) {
-    printf("Problem opening zonefile %s.\n\n",zonfile2);
+  vod = fopen(voidfile,"w");
+  if (vod == NULL) {
+    printf("Problem opening voidfile %s.\n\n",voidfile);
     exit(0);
   }
-  fwrite(&nzones,1,sizeof(int),zon2);
+  fprintf(vod,"%d\n",nzones);
 
   for (h = 0; h<nzones; h++) {
+    fprintf(vod,"%d  ",h);
+    /*if (p[z[h].core].dens > borderdens) {
+      fprintf(vod,"0 %f\n",z[h].denscontrast);
+      continue;
+      }*/
     nhlcount = 0;
     for (hl = 0; hl < nhl; hl++)
       inyet[zonelist[hl]] = 0;
@@ -413,7 +459,7 @@ int main(int argc,char **argv) {
 
       if (nl == 0) {
 	beaten = 1;
-	z[h].leak = maxvol;
+	z[h].leak = maxdens;
 	continue;
       }
 	
@@ -473,12 +519,16 @@ int main(int argc,char **argv) {
       if (beaten == 1) {
 	z[h].leak = lowvol;
       } else {
+	fprintf(vod,"%d %f ",nhl2, lowvol/p[z[h].core].dens);
 	for (h2 = 0; h2 < nhl2; h2++) {
 	  zonelist[nhl] = zonelist2[h2];
 	  inyet[zonelist2[h2]] = 1;
 	  nhl++;
 	  z[h].npjoin += z[zonelist2[h2]].np;
+	  fprintf(vod,"%d ",zonelist2[h2]);
 	}
+	fprintf(vod," ");
+	fflush(vod);
       }
       if (nhl/10000 > nhlcount) {
 	nhlcount = nhl/10000;
@@ -487,8 +537,9 @@ int main(int argc,char **argv) {
     } while((lowvol < BF) && (beaten == 0));
     
     z[h].denscontrast = z[h].leak/p[z[h].core].dens;
-    if (z[h].denscontrast < 1.) z[h].denscontrast = 1.;
     
+    fprintf(vod,"0 %f\n",z[h].denscontrast); /* Mark the end of the line */
+
     /* find biggest denscontrast */
     if (z[h].denscontrast > maxdenscontrast) {
       maxdenscontrast = (realT)z[h].denscontrast;
@@ -508,10 +559,10 @@ int main(int argc,char **argv) {
 
     z[h].nhl = nhl;
 
-    fwrite(&nhl,1,sizeof(int),zon2);
-    fwrite(zonelist,nhl,sizeof(int),zon2);
+    /*fwrite(&nhl,1,sizeof(int),vod);
+      fwrite(zonelist,nhl,sizeof(int),vod);*/
   }
-  fclose(zon2);
+  fclose(vod);
 
   printf("Maxdenscontrast = %g.\n",maxdenscontrast);
 
@@ -532,7 +583,12 @@ int main(int argc,char **argv) {
   fprintf(txt,"Void# FileVoid# CoreParticle CoreDens ZoneVol Zone#Part Void#Zones VoidVol Void#Part VoidDensContrast VoidProb\n");
   for (h=0; h<nzones; h++) {
     i = iord[h];
-    prob = exp(-5.12*(z[i].denscontrast-1.) - 0.8*pow(z[i].denscontrast-1.,2.8));
+    if (obo == 'v') { /* 3-D probability for voids */
+      prob = exp(-5.12*(z[i].denscontrast-1.) - 0.8*pow(z[i].denscontrast-1.,2.8));
+    } else { /* 3-D probability for clusters */
+      p[z[i].core].dens = 1.0/p[z[i].core].dens; /*since it was actually the volume */
+      prob = 1.077/(pow(z[i].denscontrast-1.,1.82)+0.077*pow(z[i].denscontrast,4.41));
+    }
     fprintf(txt,"%d %d %d %e %e %d %d %e %d %f %6.2e\n",
 	    h+1, i, z[i].core, p[z[i].core].dens, z[i].vol, z[i].np, z[i].nhl, z[i].voljoin, z[i].npjoin, z[i].denscontrast, prob);
 
