@@ -19,6 +19,10 @@
 #define NGUARD 42 /* Actually, the number of SPACES between guard points
 		    in each dim */
 
+/* number of particle to read in per chunk */
+/* 1024**2 */
+#define N_CHUNK 1048576
+
 /* 
    These are needed for scanning strings correctly.
 */
